@@ -5,13 +5,13 @@ Defines a request dispatcher, a HTTP request handler, a HTTP server and a
 CGI request handler.
 
 :authors: Josh Marshall, Thomas Calmant
-:copyright: Copyright 2017, Thomas Calmant
+:copyright: Copyright 2020, Thomas Calmant
 :license: Apache License 2.0
-:version: 0.3.1
+:version: 0.4.2
 
 ..
 
-    Copyright 2017 Thomas Calmant
+    Copyright 2020 Thomas Calmant
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -26,6 +26,9 @@ CGI request handler.
     limitations under the License.
 """
 
+# We use print() in the CGI request handler
+from __future__ import print_function
+
 # Standard library
 import logging
 import socket
@@ -36,21 +39,25 @@ try:
     # Python 3
     # pylint: disable=F0401,E0611
     import xmlrpc.server as xmlrpcserver
+
     # Make sure the module is complete.
     # The "future" package under python2.7 provides an incomplete
     # variant of this package.
     SimpleXMLRPCDispatcher = xmlrpcserver.SimpleXMLRPCDispatcher
     SimpleXMLRPCRequestHandler = xmlrpcserver.SimpleXMLRPCRequestHandler
-    resolve_dotted_attribute = xmlrpcserver.resolve_dotted_attribute
+    CGIXMLRPCRequestHandler = xmlrpcserver.CGIXMLRPCRequestHandler
+    resolve_dotted_attribute = xmlrpcserver.resolve_dotted_attribute  # type: ignore  # noqa: E501  # pylint: disable=invalid-name,line-too-long
     import socketserver
 except (ImportError, AttributeError):
     # Python 2 or IronPython
     # pylint: disable=F0401,E0611
-    import SimpleXMLRPCServer as xmlrpcserver
-    SimpleXMLRPCDispatcher = xmlrpcserver.SimpleXMLRPCDispatcher
-    SimpleXMLRPCRequestHandler = xmlrpcserver.SimpleXMLRPCRequestHandler
-    resolve_dotted_attribute = xmlrpcserver.resolve_dotted_attribute
-    import SocketServer as socketserver
+    import SimpleXMLRPCServer as xmlrpcserver  # type: ignore
+
+    SimpleXMLRPCDispatcher = xmlrpcserver.SimpleXMLRPCDispatcher  # type: ignore  # noqa: E501  # pylint: disable=invalid-name,line-too-long
+    SimpleXMLRPCRequestHandler = xmlrpcserver.SimpleXMLRPCRequestHandler  # type: ignore  # noqa: E501  # pylint: disable=invalid-name,line-too-long
+    CGIXMLRPCRequestHandler = xmlrpcserver.CGIXMLRPCRequestHandler  # type: ignore  # noqa: E501  # pylint: disable=invalid-name,line-too-long
+    resolve_dotted_attribute = xmlrpcserver.resolve_dotted_attribute  # type: ignore  # noqa: E501  # pylint: disable=invalid-name,line-too-long
+    import SocketServer as socketserver  # type: ignore
 
 try:
     # Windows
@@ -58,7 +65,14 @@ try:
 except ImportError:
     # Other systems
     # pylint: disable=C0103
-    fcntl = None
+    fcntl = None  # type: ignore
+
+try:
+    # Python with support for Unix socket
+    _AF_UNIX = socket.AF_UNIX
+except AttributeError:
+    # Unix sockets are not supported, use a dummy value
+    _AF_UNIX = -1  # type: ignore
 
 # Local modules
 from jsonrpclib import Fault
@@ -69,7 +83,7 @@ import jsonrpclib.threadpool
 # ------------------------------------------------------------------------------
 
 # Module version
-__version_info__ = (0, 3, 1)
+__version_info__ = (0, 4, 2)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -88,9 +102,9 @@ def get_version(request):
     :param request: A request dictionary
     :return: The JSON-RPC version or None
     """
-    if 'jsonrpc' in request:
+    if "jsonrpc" in request:
         return 2.0
-    elif 'id' in request:
+    elif "id" in request:
         return 1.0
 
     return None
@@ -106,42 +120,56 @@ def validate_request(request, json_config):
     """
     if not isinstance(request, utils.DictType):
         # Invalid request type
-        fault = Fault(-32600, 'Request must be a dict, not {0}'
-                      .format(type(request).__name__),
-                      config=json_config)
+        fault = Fault(
+            -32600,
+            "Request must be a dict, not {0}".format(type(request).__name__),
+            config=json_config,
+        )
         _logger.warning("Invalid request content: %s", fault)
         return fault
 
     # Get the request ID
-    rpcid = request.get('id', None)
+    rpcid = request.get("id", None)
 
     # Check request version
     version = get_version(request)
     if not version:
-        fault = Fault(-32600, 'Request {0} invalid.'.format(request),
-                      rpcid=rpcid, config=json_config)
+        fault = Fault(
+            -32600,
+            "Request {0} invalid.".format(request),
+            rpcid=rpcid,
+            config=json_config,
+        )
         _logger.warning("No version in request: %s", fault)
         return fault
 
     # Default parameters: empty list
-    request.setdefault('params', [])
+    request.setdefault("params", [])
 
     # Check parameters
-    method = request.get('method', None)
-    params = request.get('params')
+    method = request.get("method", None)
+    params = request.get("params")
     param_types = (utils.ListType, utils.DictType, utils.TupleType)
 
-    if not method or not isinstance(method, utils.STRING_TYPES) or \
-            not isinstance(params, param_types):
+    if (
+        not method
+        or not isinstance(method, utils.STRING_TYPES)
+        or not isinstance(params, param_types)
+    ):
         # Invalid type of method name or parameters
-        fault = Fault(-32600, 'Invalid request parameters or method.',
-                      rpcid=rpcid, config=json_config)
+        fault = Fault(
+            -32600,
+            "Invalid request parameters or method.",
+            rpcid=rpcid,
+            config=json_config,
+        )
         _logger.warning("Invalid request content: %s", fault)
         return fault
 
     # Valid request
     return True
 
+
 # ------------------------------------------------------------------------------
 
 
@@ -149,7 +177,6 @@ class NoMulticallResult(Exception):
     """
     No result in multicall
     """
-    pass
 
 
 class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
@@ -160,13 +187,15 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
     and then to dispatch them. This class doesn't need to be
     instanced directly when used by SimpleJSONRPCServer.
     """
+
     def __init__(self, encoding=None, config=jsonrpclib.config.DEFAULT):
         """
         Sets up the dispatcher with the given encoding.
         None values are allowed.
         """
         SimpleXMLRPCDispatcher.__init__(
-            self, allow_none=True, encoding=encoding or "UTF-8")
+            self, allow_none=True, encoding=encoding or "UTF-8"
+        )
         self.json_config = config
 
         # Notification thread pool
@@ -191,8 +220,11 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
         """
         if not request:
             # Invalid request dictionary
-            fault = Fault(-32600, 'Request invalid -- no request data.',
-                          config=self.json_config)
+            fault = Fault(
+                -32600,
+                "Request invalid -- no request data.",
+                config=self.json_config,
+            )
             _logger.warning("Invalid request: %s", fault)
             return fault.dump()
 
@@ -208,7 +240,8 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
 
                 # Call the method
                 resp_entry = self._marshaled_single_dispatch(
-                    req_entry, dispatch_method)
+                    req_entry, dispatch_method
+                )
 
                 # Store its result
                 if isinstance(resp_entry, Fault):
@@ -231,8 +264,7 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
                 return result.dump()
 
             # Call the method
-            response = self._marshaled_single_dispatch(
-                request, dispatch_method)
+            response = self._marshaled_single_dispatch(request, dispatch_method)
             if isinstance(response, Fault):
                 # pylint: disable=E1103
                 return response.dump()
@@ -254,9 +286,13 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
             request = jsonrpclib.loads(data, self.json_config)
         except Exception as ex:
             # Parsing/loading error
-            fault = Fault(-32700, 'Request {0} invalid. ({1}:{2})'
-                          .format(data, type(ex).__name__, ex),
-                          config=self.json_config)
+            fault = Fault(
+                -32700,
+                "Request {0} invalid. ({1}:{2})".format(
+                    data, type(ex).__name__, ex
+                ),
+                config=self.json_config,
+            )
             _logger.warning("Error parsing request: %s", fault)
             return fault.response()
 
@@ -268,10 +304,10 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
                 return jsonrpclib.jdumps(response, self.encoding)
             else:
                 # No result (notification)
-                return ''
+                return ""
         except NoMulticallResult:
             # Return an empty string (jsonrpclib internal behaviour)
-            return ''
+            return ""
 
     def _marshaled_single_dispatch(self, request, dispatch_method=None):
         """
@@ -282,11 +318,11 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
         :return: A JSON-RPC response dictionary, or None if it was a
                  notification request
         """
-        method = request.get('method')
-        params = request.get('params')
+        method = request.get("method")
+        params = request.get("params")
 
         # Prepare a request-specific configuration
-        if 'jsonrpc' not in request and self.json_config.version >= 2:
+        if "jsonrpc" not in request and self.json_config.version >= 2:
             # JSON-RPC 1.0 request on a JSON-RPC 2.0
             # => compatibility needed
             config = self.json_config.copy()
@@ -296,15 +332,17 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
             config = self.json_config
 
         # Test if this is a notification request
-        is_notification = 'id' not in request or request['id'] in (None, '')
+        is_notification = "id" not in request or request["id"] in (None, "")
         if is_notification and self.__notification_pool is not None:
             # Use the thread pool for notifications
             if dispatch_method is not None:
                 self.__notification_pool.enqueue(
-                    dispatch_method, method, params)
+                    dispatch_method, method, params
+                )
             else:
                 self.__notification_pool.enqueue(
-                    self._dispatch, method, params, config)
+                    self._dispatch, method, params, config
+                )
 
             # Return immediately
             return None
@@ -318,8 +356,11 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
                     response = self._dispatch(method, params, config)
             except Exception as ex:
                 # Return a fault
-                fault = Fault(-32603, '{0}:{1}'.format(type(ex).__name__, ex),
-                              config=config)
+                fault = Fault(
+                    -32603,
+                    "{0}:{1}".format(type(ex).__name__, ex),
+                    config=config,
+                )
                 _logger.error("Error calling method %s: %s", method, fault)
                 return fault.dump()
 
@@ -330,12 +371,14 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
 
         # Prepare a JSON-RPC dictionary
         try:
-            return jsonrpclib.dump(response, rpcid=request['id'],
-                                   is_response=True, config=config)
+            return jsonrpclib.dump(
+                response, rpcid=request["id"], is_response=True, config=config
+            )
         except Exception as ex:
             # JSON conversion exception
-            fault = Fault(-32603, '{0}:{1}'.format(type(ex).__name__, ex),
-                          config=config)
+            fault = Fault(
+                -32603, "{0}:{1}".format(type(ex).__name__, ex), config=config
+            )
             _logger.error("Error preparing JSON-RPC result: %s", fault)
             return fault.dump()
 
@@ -359,12 +402,13 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
                 # Try with the registered instance
                 try:
                     # Instance has a custom dispatcher
-                    return getattr(self.instance, '_dispatch')(method, params)
+                    return getattr(self.instance, "_dispatch")(method, params)
                 except AttributeError:
                     # Resolve the method name in the instance
                     try:
                         func = resolve_dotted_attribute(
-                            self.instance, method, True)
+                            self.instance, method, True
+                        )
                     except AttributeError:
                         # Unknown method
                         pass
@@ -378,25 +422,35 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCDispatcher, object):
                     return func(**params)
             except TypeError as ex:
                 # Maybe the parameters are wrong
-                fault = Fault(-32602, 'Invalid parameters: {0}'.format(ex),
-                              config=config)
+                fault = Fault(
+                    -32602, "Invalid parameters: {0}".format(ex), config=config
+                )
                 _logger.warning("Invalid call parameters: %s", fault)
                 return fault
             except:
                 # Method exception
                 err_lines = traceback.format_exception(*sys.exc_info())
-                trace_string = '{0} | {1}'.format(err_lines[-2].splitlines()[0].strip(), err_lines[-1])
-                fault = Fault(-32603, 'Server error: {0}'.format(trace_string),
-                              config=config)
+                trace_string = "{0} | {1}".format(
+                    err_lines[-2].splitlines()[0].strip(), err_lines[-1]
+                )
+                fault = Fault(
+                    -32603,
+                    "Server error: {0}".format(trace_string),
+                    config=config,
+                )
                 _logger.exception("Server-side exception: %s", fault)
                 return fault
         else:
             # Unknown method
-            fault = Fault(-32601, 'Method {0} not supported.'.format(method),
-                          config=config)
+            fault = Fault(
+                -32601,
+                "Method {0} not supported.".format(method),
+                config=config,
+            )
             _logger.warning("Unknown method: %s", fault)
             return fault
 
+
 # ------------------------------------------------------------------------------
 
 
@@ -407,6 +461,7 @@ class SimpleJSONRPCRequestHandler(SimpleXMLRPCRequestHandler):
     The server that receives the requests must have a json_config member,
     containing a JSONRPClib Config instance
     """
+
     def do_POST(self):
         """
         Handles POST requests
@@ -416,7 +471,7 @@ class SimpleJSONRPCRequestHandler(SimpleXMLRPCRequestHandler):
             return
 
         # Retrieve the configuration
-        config = getattr(self.server, 'json_config', jsonrpclib.config.DEFAULT)
+        config = getattr(self.server, "json_config", jsonrpclib.config.DEFAULT)
 
         try:
             # Read the request body
@@ -429,8 +484,8 @@ class SimpleJSONRPCRequestHandler(SimpleXMLRPCRequestHandler):
                 if not raw_chunk:
                     break
                 chunks.append(utils.from_bytes(raw_chunk))
-                size_remaining -= len(chunks[-1])
-            data = ''.join(chunks)
+                size_remaining -= len(raw_chunk)
+            data = "".join(chunks)
 
             try:
                 # Decode content
@@ -444,7 +499,8 @@ class SimpleJSONRPCRequestHandler(SimpleXMLRPCRequestHandler):
 
             # Execute the method
             response = self.server._marshaled_dispatch(
-                data, getattr(self, '_dispatch', None), self.path)
+                data, getattr(self, "_dispatch", None), self.path
+            )
 
             # No exception: send a 200 OK
             self.send_response(200)
@@ -452,15 +508,18 @@ class SimpleJSONRPCRequestHandler(SimpleXMLRPCRequestHandler):
             # Exception: send 500 Server Error
             self.send_response(500)
             err_lines = traceback.format_exception(*sys.exc_info())
-            trace_string = '{0} | {1}'.format(err_lines[-2].splitlines()[0].strip(), err_lines[-1])
-            fault = jsonrpclib.Fault(-32603, 'Server error: {0}'
-                                     .format(trace_string), config=config)
+            trace_string = "{0} | {1}".format(
+                err_lines[-2].splitlines()[0].strip(), err_lines[-1]
+            )
+            fault = jsonrpclib.Fault(
+                -32603, "Server error: {0}".format(trace_string), config=config
+            )
             _logger.exception("Server-side error: %s", fault)
             response = fault.response()
 
         if response is None:
             # Avoid to send None
-            response = ''
+            response = ""
 
         # Convert the response to the valid string format
         response = utils.to_bytes(response)
@@ -472,6 +531,7 @@ class SimpleJSONRPCRequestHandler(SimpleXMLRPCRequestHandler):
         if response:
             self.wfile.write(response)
 
+
 # ------------------------------------------------------------------------------
 
 
@@ -479,14 +539,21 @@ class SimpleJSONRPCServer(socketserver.TCPServer, SimpleJSONRPCDispatcher):
     """
     JSON-RPC server (and dispatcher)
     """
+
     # This simplifies server restart after error
     allow_reuse_address = True
 
     # pylint: disable=C0103
-    def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
-                 logRequests=True, encoding=None, bind_and_activate=True,
-                 address_family=socket.AF_INET,
-                 config=jsonrpclib.config.DEFAULT):
+    def __init__(
+        self,
+        addr,
+        requestHandler=SimpleJSONRPCRequestHandler,
+        logRequests=True,
+        encoding=None,
+        bind_and_activate=True,
+        address_family=socket.AF_INET,
+        config=jsonrpclib.config.DEFAULT,
+    ):
         """
         Sets up the server and the dispatcher
 
@@ -501,34 +568,53 @@ class SimpleJSONRPCServer(socketserver.TCPServer, SimpleJSONRPCDispatcher):
         # Set up the dispatcher fields
         SimpleJSONRPCDispatcher.__init__(self, encoding, config)
 
+        # Flag to ease handling of Unix socket mode
+        unix_socket = address_family == _AF_UNIX
+
+        # Disable the reuse address flag when in Unix socket mode, or an
+        # exception will raise when binding the socket
+        self.allow_reuse_address = self.allow_reuse_address and not unix_socket
+
         # Prepare the server configuration
-        # logRequests is used by SimpleXMLRPCRequestHandler
-        self.logRequests = logRequests
         self.address_family = address_family
         self.json_config = config
 
+        # logRequests is used by SimpleXMLRPCRequestHandler
+        # This must be disabled in Unix socket mode (or an exception will raise
+        # at each connection)
+        self.logRequests = logRequests and not unix_socket
+
         # Work on the request handler
         class RequestHandlerWrapper(requestHandler, object):
             """
             Wraps the request handle to have access to the configuration
             """
+
             def __init__(self, *args, **kwargs):
                 """
                 Constructs the wrapper after having stored the configuration
                 """
                 self.config = config
+
+                if unix_socket:
+                    # Disable TCP features over Unix socket, or an
+                    # "invalid argument" error will raise
+                    self.disable_nagle_algorithm = False
+
                 super(RequestHandlerWrapper, self).__init__(*args, **kwargs)
 
         # Set up the server
-        socketserver.TCPServer.__init__(self, addr, requestHandler,
-                                        bind_and_activate)
+        socketserver.TCPServer.__init__(
+            self, addr, RequestHandlerWrapper, bind_and_activate
+        )
 
         # Windows-specific
-        if fcntl is not None and hasattr(fcntl, 'FD_CLOEXEC'):
+        if fcntl is not None and hasattr(fcntl, "FD_CLOEXEC"):
             flags = fcntl.fcntl(self.fileno(), fcntl.F_GETFD)
             flags |= fcntl.FD_CLOEXEC
             fcntl.fcntl(self.fileno(), fcntl.F_SETFD, flags)
 
+
 # ------------------------------------------------------------------------------
 
 
@@ -536,10 +622,18 @@ class PooledJSONRPCServer(SimpleJSONRPCServer, socketserver.ThreadingMixIn):
     """
     JSON-RPC server based on a thread pool
     """
-    def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
-                 logRequests=True, encoding=None, bind_and_activate=True,
-                 address_family=socket.AF_INET,
-                 config=jsonrpclib.config.DEFAULT, thread_pool=None):
+
+    def __init__(
+        self,
+        addr,
+        requestHandler=SimpleJSONRPCRequestHandler,
+        logRequests=True,
+        encoding=None,
+        bind_and_activate=True,
+        address_family=socket.AF_INET,
+        config=jsonrpclib.config.DEFAULT,
+        thread_pool=None,
+    ):
         """
         Sets up the server and the dispatcher
 
@@ -556,23 +650,32 @@ class PooledJSONRPCServer(SimpleJSONRPCServer, socketserver.ThreadingMixIn):
         if thread_pool is None:
             # Start a thread pool with  30 threads max, 0 thread min
             thread_pool = jsonrpclib.threadpool.ThreadPool(
-                30, 0, logname="PooledJSONRPCServer")
+                30, 0, logname="PooledJSONRPCServer"
+            )
             thread_pool.start()
 
         # Store the thread pool
         self.__request_pool = thread_pool
 
         # Prepare the server
-        SimpleJSONRPCServer.__init__(self, addr, requestHandler, logRequests,
-                                     encoding, bind_and_activate,
-                                     address_family, config)
+        SimpleJSONRPCServer.__init__(
+            self,
+            addr,
+            requestHandler,
+            logRequests,
+            encoding,
+            bind_and_activate,
+            address_family,
+            config,
+        )
 
     def process_request(self, request, client_address):
         """
         Handle a client request: queue it in the thread pool
         """
-        self.__request_pool.enqueue(self.process_request_thread,
-                                    request, client_address)
+        self.__request_pool.enqueue(
+            self.process_request_thread, request, client_address
+        )
 
     def server_close(self):
         """
@@ -582,14 +685,18 @@ class PooledJSONRPCServer(SimpleJSONRPCServer, socketserver.ThreadingMixIn):
         SimpleJSONRPCServer.server_close(self)
         self.__request_pool.stop()
 
+
 # ------------------------------------------------------------------------------
 
 
-class CGIJSONRPCRequestHandler(SimpleJSONRPCDispatcher):
+class CGIJSONRPCRequestHandler(
+    SimpleJSONRPCDispatcher, CGIXMLRPCRequestHandler
+):
     """
     JSON-RPC CGI handler (and dispatcher)
     """
-    def __init__(self, encoding=None, config=jsonrpclib.config.DEFAULT):
+
+    def __init__(self, encoding="UTF-8", config=jsonrpclib.config.DEFAULT):
         """
         Sets up the dispatcher
 
@@ -597,17 +704,25 @@ class CGIJSONRPCRequestHandler(SimpleJSONRPCDispatcher):
         :param config: A JSONRPClib Config instance
         """
         SimpleJSONRPCDispatcher.__init__(self, encoding, config)
+        CGIXMLRPCRequestHandler.__init__(self, encoding=encoding)
 
     def handle_jsonrpc(self, request_text):
         """
         Handle a JSON-RPC request
         """
+        try:
+            writer = sys.stdout.buffer
+        except AttributeError:
+            writer = sys.stdout
+
         response = self._marshaled_dispatch(request_text)
-        sys.stdout.write('Content-Type: {0}\r\n'
-                         .format(self.json_config.content_type))
-        sys.stdout.write('Content-Length: {0:d}\r\n'.format(len(response)))
-        sys.stdout.write('\r\n')
-        sys.stdout.write(response)
+        response = response.encode(self.encoding)
+        print("Content-Type:", self.json_config.content_type)
+        print("Content-Length:", len(response))
+        print()
+        sys.stdout.flush()
+        writer.write(response)
+        writer.flush()
 
     # XML-RPC alias
     handle_xmlrpc = handle_jsonrpc
