@@ -3,24 +3,33 @@
 """
 The configuration module.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-   http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
+:copyright: Copyright 2020, Thomas Calmant
 :license: Apache License 2.0
-:version: 0.2.3
+:version: 0.4.2
+
+..
+
+    Copyright 2020 Thomas Calmant
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
+import sys
+
+# ------------------------------------------------------------------------------
+
 # Module version
-__version_info__ = (0, 2, 3)
+__version_info__ = (0, 4, 2)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -28,22 +37,21 @@ __docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
-import sys
-
-# ------------------------------------------------------------------------------
-
 
 class LocalClasses(dict):
     """
     Associates local classes with their names (used in the jsonclass module)
     """
-    def add(self, cls):
+
+    def add(self, cls, name=None):
         """
         Stores a local class
 
         :param cls: A class
+        :param name: Custom name used in the __jsonclass__ attribute
         """
-        self[cls.__name__] = cls
+        self[name or cls.__name__] = cls
+
 
 # ------------------------------------------------------------------------------
 
@@ -55,11 +63,17 @@ class Config(object):
     You can change serialize_method and ignore_attribute, or use
     the local_classes.add(class) to include "local" classes.
     """
-    def __init__(self, version=2.0, content_type="application/json-rpc",
-                 user_agent=None, use_jsonclass=True,
-                 serialize_method='_serialize',
-                 ignore_attribute='_ignore',
-                 serialize_handlers=None):
+
+    def __init__(
+        self,
+        version=2.0,
+        content_type="application/json-rpc",
+        user_agent=None,
+        use_jsonclass=True,
+        serialize_method="_serialize",
+        ignore_attribute="_ignore",
+        serialize_handlers=None,
+    ):
         """
         Sets up a configuration of JSONRPClib
 
@@ -91,9 +105,9 @@ class Config(object):
 
         # Default user agent
         if user_agent is None:
-            user_agent = 'jsonrpclib/{0} (Python {1})'.format(
-                __version__, '.'.join(str(ver)
-                                      for ver in sys.version_info[0:3]))
+            user_agent = "jsonrpclib/{0} (Python {1})".format(
+                __version__, ".".join(str(ver) for ver in sys.version_info[0:3])
+            )
         self.user_agent = user_agent
 
         # The list of classes to use for jsonclass translation.
@@ -122,9 +136,19 @@ class Config(object):
 
         :return: A shallow copy of this configuration
         """
-        return Config(self.version, self.content_type, self.user_agent,
-                      self.use_jsonclass, self.serialize_method,
-                      self.ignore_attribute, self.serialize_handlers)
+        new_config = Config(
+            self.version,
+            self.content_type,
+            self.user_agent,
+            self.use_jsonclass,
+            self.serialize_method,
+            self.ignore_attribute,
+            None,
+        )
+        new_config.classes = self.classes.copy()
+        new_config.serialize_handlers = self.serialize_handlers.copy()
+        return new_config
+
 
 # Default configuration
 DEFAULT = Config()
