@@ -3,24 +3,30 @@
 """
 The history module.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-   http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
+:authors: Josh Marshall, Thomas Calmant
+:copyright: Copyright 2020, Thomas Calmant
 :license: Apache License 2.0
-:version: 0.2.3
+:version: 0.4.2
+
+..
+
+    Copyright 2020 Thomas Calmant
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 """
 
 # Module version
-__version_info__ = (0, 2, 3)
+__version_info__ = (0, 4, 2)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -36,6 +42,7 @@ class History(object):
     each request cycle in order to keep it from clogging
     memory.
     """
+
     def __init__(self):
         """
         Sets up members
@@ -66,7 +73,6 @@ class History(object):
         """
         try:
             return self.requests[-1]
-
         except IndexError:
             return None
 
@@ -77,7 +83,6 @@ class History(object):
         """
         try:
             return self.responses[-1]
-
         except IndexError:
             return None
 
